/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var paramEditor = (function(paramEditor) {

  // entry function
  window.addEventListener("load", function(evt) {
    var tabs = document.querySelectorAll(".tab");
    var tabpages = document.querySelectorAll(".tabpage");
    var tabs_container = document.querySelector("#tabs");
    var current = tabs_container.getAttribute("data-current");

    for (var i=0, l=tabs.length; i<l; i++) {
      if (current == i) {
        tabs[i].setAttribute("class", "tabActiveHeader");
        tabpages[i].style.display = "block";
      }
      tabs[i].addEventListener("click", displayPage);
    }

    /**
     *
     */
    function displayPage() {
      tabs[current].setAttribute("class", "tab");
      tabpages[current].style.display = "none";
      current = this.id.split("_")[1];

      tabs[current].setAttribute("class", "tabActiveHeader");
      tabpages[current].style.display = "block";
    }

    paramEditor.initPanels();

    var params = document.querySelector("ajs");

    paramEditor.setParams(params);
  });

  paramEditor.initPanels = function() {
    // 🔘○◯

    this.buttonsPanel = new paramEditor.PanelButtons();
    document.querySelector("#tabpage_buttons_0 .full_panel").appendChild(this.buttonsPanel.container);

    //
    this.spacesPanelListEdit = new paramEditor.PanelListEdit("Space", false);
    document.querySelector("#tabpage_spaces_1 .left_panel").appendChild(this.spacesPanelListEdit.container);
    this.spacesPanel = new paramEditor.PanelSpaces();
    document.querySelector("#tabpage_spaces_1 .right_panel").appendChild(this.spacesPanel.container);

    this.spacesPanelListEdit.setEditPanel(this.spacesPanel);
    this.spacesPanel.setEditPanel(this.spacesPanelListEdit);

    //
    this.controlsPanelListEdit = new paramEditor.PanelListEdit("Controls", true);
    document.querySelector("#tabpage_controls_2 .left_panel").appendChild(this.controlsPanelListEdit.container);
    this.controlsPanel = new paramEditor.PanelControls();
    document.querySelector("#tabpage_controls_2 .right_panel").appendChild(this.controlsPanel.container);

    this.controlsPanelListEdit.setEditPanel(this.controlsPanel);
    this.controlsPanel.setEditPanel(this.controlsPanelListEdit);

    //
    this.definitionsPanelListEdit = new paramEditor.PanelListEdit("Definitions", false);
    document.querySelector("#tabpage_definitions_3 .left_panel").appendChild(this.definitionsPanelListEdit.container);
    this.definitionsPanel = new paramEditor.PanelDefinitions();
    document.querySelector("#tabpage_definitions_3 .right_panel").appendChild(this.definitionsPanel.container);

    this.definitionsPanelListEdit.setEditPanel(this.definitionsPanel);
    this.definitionsPanel.setEditPanel(this.definitionsPanelListEdit);

    //
    this.programsPanelListEdit = new paramEditor.PanelListEdit("Programs", false);
    document.querySelector("#tabpage_programs_4 .left_panel").appendChild(this.programsPanelListEdit.container);
    this.programsPanel = new paramEditor.PanelPrograms();
    document.querySelector("#tabpage_programs_4 .right_panel").appendChild(this.programsPanel.container);

    this.programsPanelListEdit.setEditPanel(this.programsPanel);
    this.programsPanel.setEditPanel(this.programsPanelListEdit);

    //
    this.graphicsPanelListEdit = new paramEditor.PanelListEdit("Graphics", true);
    document.querySelector("#tabpage_graphics_5 .left_panel").appendChild(this.graphicsPanelListEdit.container);
    this.graphicsPanel = new paramEditor.PanelGraphics();
    document.querySelector("#tabpage_graphics_5 .right_panel").appendChild(this.graphicsPanel.container);

    this.graphicsPanelListEdit.setEditPanel(this.graphicsPanel);
    this.graphicsPanel.setEditPanel(this.graphicsPanelListEdit);

    //
    this.graphics3DPanelListEdit = new paramEditor.PanelListEdit("Graphics 3D", true);
    document.querySelector("#tabpage_graphics3D_6 .left_panel").appendChild(this.graphics3DPanelListEdit.container);
    this.graphics3DPanel = new paramEditor.PanelGraphics3D();
    document.querySelector("#tabpage_graphics3D_6 .right_panel").appendChild(this.graphics3DPanel.container);

    this.graphics3DPanelListEdit.setEditPanel(this.graphics3DPanel);
    this.graphics3DPanel.setEditPanel(this.graphics3DPanelListEdit);

    //
    this.animationPanel = new paramEditor.PanelAnimation();
    document.querySelector("#tabpage_animation_7 .full_panel").appendChild(this.animationPanel.container);
  }

  paramEditor.setParams = function(param) {
    this.model = new paramEditor.Model(param);

    // set the appropiated model to the editor
    this.buttonsPanel.setModelObj(this.model.data.attributes);
    this.spacesPanelListEdit.setModelObj(this.model, "spaces");
    this.controlsPanelListEdit.setModelObj(this.model, "controls");
    this.definitionsPanelListEdit.setModelObj(this.model, "definitions");
    this.programsPanelListEdit.setModelObj(this.model, "programs");
    this.graphicsPanelListEdit.setModelObj(this.model, "graphics");
    this.graphics3DPanelListEdit.setModelObj(this.model, "graphics3D");
    this.animationPanel.setModelObj(this.model.data.animation);

    // console.log(this.model);
  }

  return paramEditor;
})(paramEditor || {});
