/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var paramEditor = (function(paramEditor) {

  /**
   *
   */
  paramEditor.Panel = function(type) {
    var obj = { 
      container: document.createElement("div"),
      components: {}
    }
    obj.container.setAttribute("class", "panel");

    switch (type) {
      case "buttons":
        this.makePanelButtons(obj);
        break;

      case "spaces":
        this.makePanelSpaces(obj);
        break;

      case "controls":
        this.makePanelControls(obj);
        break;

      case "definitions":
        this.makePanelDefinitions(obj);
        break;

      case "programs":
        this.makePanelPrograms(obj);
        break;

      case "graphics":
        this.makePanelGraphics(obj);
        break;

      case "graphics3D":
        this.makePanelGraphics3D(obj);
        break;

      case "animation":
        this.makePanelAnimation(obj);
        break;

      default:
        console.log("error: ", type);
    }

    return obj;
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelButtons = function(obj) {
    // about component
    obj.components.about = new paramEditor.LabelAndCheckbox("about", false);
    obj.container.appendChild(obj.components.about.domObj);

    // config component
    obj.components.config = new paramEditor.LabelAndCheckbox("config", false);
    obj.container.appendChild(obj.components.config.domObj);

    // init component
    obj.components.init = new paramEditor.LabelAndCheckbox("init", false);
    obj.container.appendChild(obj.components.init.domObj);

    // clear component
    obj.components.clear = new paramEditor.LabelAndCheckbox("clear", false);
    obj.container.appendChild(obj.components.clear.domObj);

    // editor component
    obj.components.editor = new paramEditor.LabelAndCheckbox("editor", false);
    obj.container.appendChild(obj.components.editor.domObj);

    obj.container.appendChild(document.createElement("br"));

    // rows_north component
    obj.components.rows_north = new paramEditor.LabelAndTextfield("rows_north", 100, 0);
    obj.container.appendChild(obj.components.rows_north.domObj);

    // rows_south component
    obj.components.rows_south = new paramEditor.LabelAndTextfield("rows_south", 100, 0);
    obj.container.appendChild(obj.components.rows_south.domObj);

    // width_west component
    obj.components.width_west = new paramEditor.LabelAndTextfield("width_west", 100, 150);
    obj.container.appendChild(obj.components.width_west.domObj);

    // width_east component
    obj.components.width_east = new paramEditor.LabelAndTextfield("width_east", 100, 150);
    obj.container.appendChild(obj.components.width_east.domObj);

    // height component
    obj.components.height = new paramEditor.LabelAndTextfield("height", 100, 40);
    obj.container.appendChild(obj.components.height.domObj);

    obj.container.appendChild(document.createElement("br"));

    // decimal_symbol
    obj.components.decimal_symbol = new paramEditor.LabelAndMenu("decimal_symbol", [",", "."], ",");
    obj.container.appendChild(obj.components.decimal_symbol.domObj);

    // language
    obj.components.language = new paramEditor.LabelAndMenu("language", ["english", "español"], "english");
    obj.container.appendChild(obj.components.language.domObj);
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelSpaces = function(obj) {
    // id component
    obj.components.id = new paramEditor.LabelAndTextfield("id", 100, "E1");
    obj.container.appendChild(obj.components.id.domObj);

    // left component
    obj.components.left = new paramEditor.LabelAndTextfield("left", 100, 0);
    obj.container.appendChild(obj.components.left.domObj);

    // top component
    obj.components.top = new paramEditor.LabelAndTextfield("top", 100, 0);
    obj.container.appendChild(obj.components.top.domObj);

    // width component
    obj.components.width = new paramEditor.LabelAndTextfield("width", 100, "100%");
    obj.container.appendChild(obj.components.width.domObj);

    // height component
    obj.components.height = new paramEditor.LabelAndTextfield("height", 100, "100%");
    obj.container.appendChild(obj.components.height.domObj);

    // draw_if component
    obj.components.draw_if = new paramEditor.LabelAndTextfield("draw_if", 150, 1);
    obj.container.appendChild(obj.components.draw_if.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // fixed component
    obj.components.fixed = new paramEditor.LabelAndCheckbox("fixed", false);
    obj.container.appendChild(obj.components.fixed.domObj);

    // scale component
    obj.components.scale = new paramEditor.LabelAndTextfield("scale", 100, 48);
    obj.container.appendChild(obj.components.scale.domObj);

    // Ox component
    obj.components.Ox = new paramEditor.LabelAndTextfield("Ox", 100, 0);
    obj.container.appendChild(obj.components.Ox.domObj);

    // Oy component
    obj.components.Oy = new paramEditor.LabelAndTextfield("Oy", 100, 0);
    obj.container.appendChild(obj.components.Oy.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    // bg_image component
    obj.components.bg_image = new paramEditor.LabelAndTextfield("bg_image", 100, "");
    obj.container.appendChild(obj.components.bg_image.domObj);

    // bg_display component
    obj.components.bg_display = new paramEditor.LabelAndMenu("bg_display", ["topleft", "stretch", "patch", "center"], "topleft");
    obj.container.appendChild(obj.components.bg_display.domObj);

    // background component
    obj.components.background = new paramEditor.LabelAndColor("background", "");
    obj.container.appendChild(obj.components.background.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // net component
    obj.components.net = new paramEditor.CheckboxAndLabelAndColor("net", true, "");
    obj.container.appendChild(obj.components.net.domObj);

    // net10 component
    obj.components.net10 = new paramEditor.CheckboxAndLabelAndColor("net10", true, "");
    obj.container.appendChild(obj.components.net10.domObj);

    // axes component
    obj.components.axes = new paramEditor.CheckboxAndLabelAndColor("axes", true, "");
    obj.container.appendChild(obj.components.axes.domObj);

    // text component
    obj.components.text = new paramEditor.CheckboxAndLabelAndColor("text", true, "");
    obj.container.appendChild(obj.components.text.domObj);

    // numbers component
    obj.components.numbers = new paramEditor.LabelAndCheckbox("numbers", false);
    obj.container.appendChild(obj.components.numbers.domObj);

    // x_axis component
    obj.components.x_axis = new paramEditor.LabelAndTextfield("x_axis", 150, "");
    obj.container.appendChild(obj.components.x_axis.domObj);

    // y_axis component
    obj.components.y_axis = new paramEditor.LabelAndTextfield("y_axis", 150, "");
    obj.container.appendChild(obj.components.y_axis.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // render component
    obj.components.render = new paramEditor.LabelAndMenu("render", ["sort", "painter", "ray trace"], "sort");
    obj.container.appendChild(obj.components.render.domObj);

    // split component
    obj.components.split = new paramEditor.LabelAndCheckbox("split", false);
    obj.container.appendChild(obj.components.split.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // sensitive_to_mouse_movements component
    obj.components.sensitive_to_mouse_movements = new paramEditor.LabelAndCheckbox("sensitive_to_mouse_movements", false);
    obj.container.appendChild(obj.components.sensitive_to_mouse_movements.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for App & HTMLIfrae spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // file component
    obj.components.file = new paramEditor.LabelAndTextfield("file", 300, "");
    obj.container.appendChild(obj.components.file.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for App & HTMLIfrae spaces
    ////////////////////////////////////////////////////////////////////////////////////
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelControls = function(obj) {
    ////////////////////////////////////////////////////////////////////////////////////////////
    // NUMERIC CONTROLS
    ////////////////////////////////////////////////////////////////////////////////////////////
    // id component
    obj.components.id = new paramEditor.LabelAndTextfield("id", 100, "n1");
    obj.container.appendChild(obj.components.id.domObj);

    // gui component
    obj.components.gui = new paramEditor.LabelAndMenu("gui", ["spinner", "textfield", "choice", "scrollbar", "button"], "spinner");
    obj.container.appendChild(obj.components.gui.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for textfield controls
    ////////////////////////////////////////////////////////////////////////////////////
    // only_text component
    obj.components.only_text = new paramEditor.LabelAndCheckbox("only_text", false);
    obj.container.appendChild(obj.components.only_text.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for textfield controls
    ////////////////////////////////////////////////////////////////////////////////////

    // region component
    obj.components.region = new paramEditor.LabelAndMenu("region", ["south", "north", "east", "west", "external", "interior", "scenario"], "south");
    obj.container.appendChild(obj.components.region.domObj);

    // space component
    obj.components.space = new paramEditor.LabelAndMenu("space", [], "spinner");
    obj.container.appendChild(obj.components.space.domObj);

    // name component
    obj.components.name = new paramEditor.LabelAndTextfield("name", 100, "n1");
    obj.container.appendChild(obj.components.name.domObj);

    // pos component
    obj.components.pos = new paramEditor.LabelAndTextfield("pos", 100, "(0,0)");
    obj.container.appendChild(obj.components.pos.domObj);

    // value component
    obj.components.value = new paramEditor.LabelAndTextfield("value", 100, "0");
    obj.container.appendChild(obj.components.value.domObj);

    // incr component
    obj.components.incr = new paramEditor.LabelAndTextfield("incr", 100, "0.1");
    obj.container.appendChild(obj.components.incr.domObj);

    // inf component
    obj.components.inf = new paramEditor.LabelAndTextfield("inf", 100, "");
    obj.container.appendChild(obj.components.inf.domObj);

    // sup component
    obj.components.sup = new paramEditor.LabelAndTextfield("sup", 100, "");
    obj.container.appendChild(obj.components.sup.domObj);

    // discrete component
    obj.components.discrete = new paramEditor.LabelAndCheckbox("discrete", false);
    obj.container.appendChild(obj.components.discrete.domObj);

    // decimals component
    obj.components.decimals = new paramEditor.LabelAndTextfield("decimals", 100, "2");
    obj.container.appendChild(obj.components.decimals.domObj);

    // fixed component
    obj.components.fixed = new paramEditor.LabelAndCheckbox("fixed", true);
    obj.container.appendChild(obj.components.fixed.domObj);

    // exponential_if component
    obj.components.exponential_if = new paramEditor.LabelAndTextfield("exponential_if", 100, "");
    obj.container.appendChild(obj.components.exponential_if.domObj);

    // visible component
    obj.components.visible = new paramEditor.LabelAndCheckbox("visible", true);
    obj.container.appendChild(obj.components.visible.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for choice controls
    ////////////////////////////////////////////////////////////////////////////////////
    // options component
    obj.components.options = new paramEditor.LabelAndTextfield("options", 500, "");
    obj.container.appendChild(obj.components.options.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for choice controls
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for button controls
    ////////////////////////////////////////////////////////////////////////////////////
    // colour component
    obj.components.colour = new paramEditor.LabelAndColor("colour", "");
    obj.container.appendChild(obj.components.colour.domObj);

    // int_colour component
    obj.components.int_colour = new paramEditor.LabelAndColor("int_colour", "");
    obj.container.appendChild(obj.components.int_colour.domObj);

    // bold component
    obj.components.bold = new paramEditor.LabelAndCheckbox("bold", false);
    obj.container.appendChild(obj.components.bold.domObj);

    // italics component
    obj.components.italics = new paramEditor.LabelAndCheckbox("italics", false);
    obj.container.appendChild(obj.components.italics.domObj);

    // underline component
    obj.components.underline = new paramEditor.LabelAndCheckbox("underline", false);
    obj.container.appendChild(obj.components.underline.domObj);

    // font_size component
    obj.components.font_size = new paramEditor.LabelAndTextfield("font_size", 100, "");
    obj.container.appendChild(obj.components.font_size.domObj);

    // image component
    obj.components.image = new paramEditor.LabelAndTextfield("image", 200, "");
    obj.container.appendChild(obj.components.image.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for button controls
    ////////////////////////////////////////////////////////////////////////////////////

    // action component
    obj.components.action = new paramEditor.LabelAndMenu("action", ["", "calculate", "init", "clear", "animate", "open URL", "open Scene", "play"], "");
    obj.container.appendChild(obj.components.action.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // parameter component
    obj.components.parameter = new paramEditor.LabelAndTextfield("parameter", 250, "");
    obj.container.appendChild(obj.components.parameter.domObj);

    // draw_if component
    obj.components.draw_if = new paramEditor.LabelAndTextfield("draw_if", 100, 1);
    obj.container.appendChild(obj.components.draw_if.domObj);

    // active_if component
    obj.components.active_if = new paramEditor.LabelAndTextfield("active_if", 100, 1);
    obj.container.appendChild(obj.components.active_if.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for textfield controls
    ////////////////////////////////////////////////////////////////////////////////////
    // evaluate component
    obj.components.evaluate = new paramEditor.LabelAndCheckbox("evaluate", false);
    obj.container.appendChild(obj.components.evaluate.domObj);

    // answer component
    obj.components.answer = new paramEditor.LabelAndTextfield("answer", 100, "");
    obj.container.appendChild(obj.components.answer.domObj);

    // weight component
    obj.components.weight = new paramEditor.LabelAndTextfield("weight", 100, "");
    obj.container.appendChild(obj.components.weight.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for textfield controls
    ////////////////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////////////////////////
    // GRAPHIC CONTROLS
    ////////////////////////////////////////////////////////////////////////////////////////////
    // size component
    obj.components.size = new paramEditor.LabelAndTextfield("size", 100, "");
    obj.container.appendChild(obj.components.size.domObj);

    // constraint component
    obj.components.constraint = new paramEditor.LabelAndTextfield("constraint", 300, "");
    obj.container.appendChild(obj.components.constraint.domObj);


    ////////////////////////////////////////////////////////////////////////////////////////////
    // TEXT CONTROLS (maybe removed)
    ////////////////////////////////////////////////////////////////////////////////////////////
    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // text component
    obj.components.text = new paramEditor.LabelAndTextfield("text", 250, "");
    obj.container.appendChild(obj.components.text.domObj);

    // Buttons component
    obj.components.Buttons = new paramEditor.LabelAndCheckbox("Buttons", false);
    obj.container.appendChild(obj.components.Buttons.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // answer component
    obj.components.answer = new paramEditor.LabelAndTextfield("answer", 250, "");
    obj.container.appendChild(obj.components.answer.domObj);


    ////////////////////////////////////////////////////////////////////////////////////////////
    // AUDIO & VIDEO CONTROLS (maybe removed)
    ////////////////////////////////////////////////////////////////////////////////////////////
    // file component
    obj.components.file = new paramEditor.LabelAndTextfield("file", 250, "");
    obj.container.appendChild(obj.components.file.domObj);
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelDefinitions = function(obj) {
    // id component
    obj.components.id = new paramEditor.LabelAndTextfield("id", 100, "");
    obj.container.appendChild(obj.components.id.domObj);

    obj.components.equalsign = document.createElement("span");
    obj.components.equalsign.innerHTML = " = ";
    obj.container.appendChild(obj.components.equalsign);

    // expresion component
    obj.components.expresion = new paramEditor.LabelAndTextfield("expresion", 300, "");
    obj.container.appendChild(obj.components.expresion.domObj);

    // range component
    obj.components.range = new paramEditor.LabelAndTextfield("range", 300, "");
    obj.container.appendChild(obj.components.range.domObj);

    // algorithm component
    obj.components.algorithm = new paramEditor.LabelAndCheckbox("algorithm", false);
    obj.container.appendChild(obj.components.algorithm.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // Local component
    obj.components.Local = new paramEditor.LabelAndTextfield("Local", 250, "");
    obj.container.appendChild(obj.components.Local.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // init component
    obj.components.init = new paramEditor.LabelAndTextfield("init", 250, "");
    obj.container.appendChild(obj.components.init.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente hacer un text area
    // do component
    obj.components.do = new paramEditor.LabelAndTextfield("do", 250, "");
    obj.container.appendChild(obj.components.do.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // while component
    obj.components.while = new paramEditor.LabelAndTextfield("while", 250, "");
    obj.container.appendChild(obj.components.while.domObj);

    // evaluate component
    obj.components.evaluate = new paramEditor.LabelAndMenu("evaluate", ["only-once"], "only-once");
    obj.container.appendChild(obj.components.evaluate.domObj);

    // size component
    obj.components.size = new paramEditor.LabelAndTextfield("size", 100, "");
    obj.container.appendChild(obj.components.size.domObj);

    // columns component
    obj.components.columns = new paramEditor.LabelAndTextfield("columns", 100, "");
    obj.container.appendChild(obj.components.columns.domObj);

    // rows component
    obj.components.rows = new paramEditor.LabelAndTextfield("rows", 100, "");
    obj.container.appendChild(obj.components.rows.domObj);

    // file component
    obj.components.file = new paramEditor.LabelAndTextfield("file", 250, "");
    obj.container.appendChild(obj.components.file.domObj);
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelPrograms = function(obj) {
    // id component
    obj.components.id = new paramEditor.LabelAndTextfield("id", 100, "");
    obj.container.appendChild(obj.components.id.domObj);

    // evaluate component
    obj.components.evaluate = new paramEditor.LabelAndMenu("evaluate", ["only-once", "always"], "only-once");
    obj.container.appendChild(obj.components.evaluate.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // init component
    obj.components.init = new paramEditor.LabelAndTextfield("init", 250, "");
    obj.container.appendChild(obj.components.init.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente hacer un text area
    // do component
    obj.components.do = new paramEditor.LabelAndTextfield("do", 250, "");
    obj.container.appendChild(obj.components.do.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // while component
    obj.components.while = new paramEditor.LabelAndTextfield("while", 250, "");
    obj.container.appendChild(obj.components.while.domObj);

    // condition component
    obj.components.condition = new paramEditor.LabelAndTextfield("condition", 250, "");
    obj.container.appendChild(obj.components.condition.domObj);

    // action component
    obj.components.action = new paramEditor.LabelAndMenu("action", ["", "calculate", "open URL", "open Scene", "init", "clear", "animate", "Init Animation", "play"], "");
    obj.container.appendChild(obj.components.action.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // parameter component
    obj.components.parameter = new paramEditor.LabelAndTextfield("parameter", 250, "");
    obj.container.appendChild(obj.components.parameter.domObj);

    // execution component
    obj.components.execution = new paramEditor.LabelAndMenu("execution", ["only-once", "alternate", "always"], "only-once");
    obj.container.appendChild(obj.components.execution.domObj);

  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelGraphics = function(obj) {
    // space component
    obj.components.space = new paramEditor.LabelAndMenu("space", [], "spinner");
    obj.container.appendChild(obj.components.space.domObj);

    // background component
    obj.components.background = new paramEditor.LabelAndCheckbox("background", false);
    obj.container.appendChild(obj.components.background.domObj);

    // colour component
    obj.components.colour = new paramEditor.LabelAndColor("colour", "");
    obj.container.appendChild(obj.components.colour.domObj);

    // draw_if component
    obj.components.draw_if = new paramEditor.LabelAndTextfield("draw_if", 100, 1);
    obj.container.appendChild(obj.components.draw_if.domObj);

    // abs_coords component
    obj.components.abs_coords = new paramEditor.LabelAndCheckbox("abs_coords", false);
    obj.container.appendChild(obj.components.abs_coords.domObj);


    // center component
    obj.components.center = new paramEditor.LabelAndTextfield("center", 100, "(0,0)");
    obj.container.appendChild(obj.components.center.domObj);

    // radius component
    obj.components.radius = new paramEditor.LabelAndTextfield("radius", 100, 1);
    obj.container.appendChild(obj.components.radius.domObj);

    // init component
    obj.components.init = new paramEditor.LabelAndTextfield("init", 100, 0);
    obj.container.appendChild(obj.components.init.domObj);

    // end component
    obj.components.end = new paramEditor.LabelAndTextfield("end", 100, 90);
    obj.container.appendChild(obj.components.end.domObj);

    // vector component
    obj.components.vector = new paramEditor.LabelAndCheckbox("vector", false);
    obj.container.appendChild(obj.components.vector.domObj);


    // expresion component
    obj.components.expresion = new paramEditor.LabelAndTextfield("expresion", 300, "");
    obj.container.appendChild(obj.components.expresion.domObj);

    // trace component
    obj.components.trace = new paramEditor.CheckboxAndLabelAndColor("trace", false, "");
    obj.container.appendChild(obj.components.trace.domObj);

    // family component
    obj.components.family = new paramEditor.LabelAndCheckbox("family", false);
    obj.container.appendChild(obj.components.family.domObj);

    // family_parameter component
    obj.components.family_parameter = new paramEditor.LabelAndTextfield("family_parameter", 100, "s");
    obj.container.appendChild(obj.components.family_parameter.domObj);

    // family_interval component
    obj.components.family_interval = new paramEditor.LabelAndTextfield("family_interval", 100, "[0,1]");
    obj.container.appendChild(obj.components.family_interval.domObj);

    // family_steps component
    obj.components.family_steps = new paramEditor.LabelAndTextfield("family_steps", 100, 8);
    obj.container.appendChild(obj.components.family_steps.domObj);

    // parameter component
    obj.components.parameter = new paramEditor.LabelAndTextfield("parameter", 100, "s");
    obj.container.appendChild(obj.components.parameter.domObj);

    // parameter_interval component
    obj.components.parameter_interval = new paramEditor.LabelAndTextfield("parameter_interval", 100, "[0,1]");
    obj.container.appendChild(obj.components.parameter_interval.domObj);

    // parameter_steps component
    obj.components.parameter_steps = new paramEditor.LabelAndTextfield("parameter_steps", 100, 8);
    obj.container.appendChild(obj.components.parameter_steps.domObj);

    // file component
    obj.components.file = new paramEditor.LabelAndTextfield("file", 200, "");
    obj.container.appendChild(obj.components.file.domObj);


    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // text component
    obj.components.text = new paramEditor.LabelAndTextfield("text", 250, "");
    obj.container.appendChild(obj.components.text.domObj);

    // decimals component
    obj.components.decimals = new paramEditor.LabelAndTextfield("decimals", 250, 2);
    obj.container.appendChild(obj.components.decimals.domObj);

    // fixed component
    obj.components.fixed = new paramEditor.LabelAndCheckbox("fixed", false);
    obj.container.appendChild(obj.components.fixed.domObj);

    // fill component
    obj.components.fill = new paramEditor.CheckboxAndLabelAndColor("fill", false, "");
    obj.container.appendChild(obj.components.fill.domObj);

    // fill_plus component
    obj.components.fill_plus = new paramEditor.CheckboxAndLabelAndColor("fill_plus", false, "");
    obj.container.appendChild(obj.components.fill_plus.domObj);

    // fill_minus component
    obj.components.fill_minus = new paramEditor.CheckboxAndLabelAndColor("fill_minus", false, "");
    obj.container.appendChild(obj.components.fill_minus.domObj);

    // size component
    obj.components.size = new paramEditor.LabelAndTextfield("size", 100, 2);
    obj.container.appendChild(obj.components.size.domObj);

    // width component
    obj.components.width = new paramEditor.LabelAndTextfield("width", 100, 1);
    obj.container.appendChild(obj.components.width.domObj);

    // spear component
    obj.components.spear = new paramEditor.LabelAndTextfield("spear", 100, 8);
    obj.container.appendChild(obj.components.spear.domObj);

    // arrow component
    obj.components.arrow = new paramEditor.LabelAndColor("arrow", "");
    obj.container.appendChild(obj.components.arrow.domObj);

    // visible component
    obj.components.visible = new paramEditor.LabelAndCheckbox("visible", false);
    obj.container.appendChild(obj.components.visible.domObj);

    // editable component
    obj.components.editable = new paramEditor.LabelAndCheckbox("editable", false);
    obj.container.appendChild(obj.components.editable.domObj);

    // range component
    obj.components.range = new paramEditor.LabelAndTextfield("range", 100, "[1,100]");
    obj.container.appendChild(obj.components.range.domObj);

    // inirot component
    obj.components.inirot = new paramEditor.LabelAndTextfield("inirot", 100, "0");
    obj.container.appendChild(obj.components.inirot.domObj);

    // inipos component
    obj.components.inipos = new paramEditor.LabelAndTextfield("inipos", 100, "[0,0]");
    obj.container.appendChild(obj.components.inipos.domObj);

    // name component
    obj.components.name = new paramEditor.LabelAndTextfield("name", 100, "");
    obj.container.appendChild(obj.components.name.domObj);

    // info component
    obj.components.info = new paramEditor.LabelAndTextfield("info", 100, "");
    obj.container.appendChild(obj.components.info.domObj);
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelGraphics3D = function(obj) {
    // name component
    obj.components.name = new paramEditor.LabelAndTextfield("name", 100, "");
    obj.container.appendChild(obj.components.name.domObj);

    // space component
    obj.components.space = new paramEditor.LabelAndMenu("space", [], "spinner");
    obj.container.appendChild(obj.components.space.domObj);

    // background component
    obj.components.background = new paramEditor.LabelAndCheckbox("background", false);
    obj.container.appendChild(obj.components.background.domObj);

    // colour component
    obj.components.colour = new paramEditor.LabelAndColor("colour", "");
    obj.container.appendChild(obj.components.colour.domObj);

    // backcolor component
    obj.components.backcolor = new paramEditor.LabelAndColor("backcolor", "");
    obj.container.appendChild(obj.components.backcolor.domObj);

    // draw_if component
    obj.components.draw_if = new paramEditor.LabelAndTextfield("draw_if", 100, 1);
    obj.container.appendChild(obj.components.draw_if.domObj);

    // expresion component
    obj.components.expresion = new paramEditor.LabelAndTextfield("expresion", 300, "");
    obj.container.appendChild(obj.components.expresion.domObj);

    // family component
    obj.components.family = new paramEditor.LabelAndCheckbox("family", false);
    obj.container.appendChild(obj.components.family.domObj);

    // family_parameter component
    obj.components.family_parameter = new paramEditor.LabelAndTextfield("family_parameter", 100, "s");
    obj.container.appendChild(obj.components.family_parameter.domObj);

    // family_interval component
    obj.components.family_interval = new paramEditor.LabelAndTextfield("family_interval", 100, "[0,1]");
    obj.container.appendChild(obj.components.family_interval.domObj);

    // family_steps component
    obj.components.family_steps = new paramEditor.LabelAndTextfield("family_steps", 100, 8);
    obj.container.appendChild(obj.components.family_steps.domObj);

    // inirot component
    obj.components.inirot = new paramEditor.LabelAndTextfield("inirot", 100, "(0,0,0)");
    obj.container.appendChild(obj.components.inirot.domObj);

    // inipos component
    obj.components.inipos = new paramEditor.LabelAndTextfield("inipos", 100, "(0,0,0)");
    obj.container.appendChild(obj.components.inipos.domObj);

    // finrot component
    obj.components.finrot = new paramEditor.LabelAndTextfield("finrot", 100, "(0,0,0)");
    obj.container.appendChild(obj.components.finrot.domObj);

    // finpos component
    obj.components.finpos = new paramEditor.LabelAndTextfield("finpos", 100, "(0,0,0)");
    obj.container.appendChild(obj.components.finpos.domObj);

    // split component
    obj.components.split = new paramEditor.LabelAndCheckbox("split", false);
    obj.container.appendChild(obj.components.split.domObj);

    // edges component
    obj.components.edges = new paramEditor.LabelAndCheckbox("edges", false);
    obj.container.appendChild(obj.components.edges.domObj);

    // model component
    obj.components.model = new paramEditor.LabelAndMenu("model", ["color", "light", "metal", "wire"], "color");
    obj.container.appendChild(obj.components.model.domObj);

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // text component
    obj.components.text = new paramEditor.LabelAndTextfield("text", 250, "");
    obj.container.appendChild(obj.components.text.domObj);

    // decimals component
    obj.components.decimals = new paramEditor.LabelAndTextfield("decimals", 250, 2);
    obj.container.appendChild(obj.components.decimals.domObj);

    // fixed component
    obj.components.fixed = new paramEditor.LabelAndCheckbox("fixed", false);
    obj.container.appendChild(obj.components.fixed.domObj);

    // width component
    obj.components.width = new paramEditor.LabelAndTextfield("width", 100, 1);
    obj.container.appendChild(obj.components.width.domObj);

    // depth component
    obj.components.depth = new paramEditor.LabelAndTextfield("depth", 100, 2);
    obj.container.appendChild(obj.components.depth.domObj);

    // length component
    obj.components.length = new paramEditor.LabelAndTextfield("length", 100, 2);
    obj.container.appendChild(obj.components.length.domObj);

    // height component
    obj.components.height = new paramEditor.LabelAndTextfield("height", 100, 2);
    obj.container.appendChild(obj.components.height.domObj);

    // Nu component
    obj.components.Nu = new paramEditor.LabelAndTextfield("Nu", 100, 7);
    obj.container.appendChild(obj.components.Nu.domObj);

    // Nv component
    obj.components.Nv = new paramEditor.LabelAndTextfield("Nv", 100, 7);
    obj.container.appendChild(obj.components.Nv.domObj);

    // info component
    obj.components.info = new paramEditor.LabelAndTextfield("info", 100, "");
    obj.container.appendChild(obj.components.info.domObj);
  }

  /**
   *
   */
  paramEditor.Panel.prototype.makePanelAnimation = function(obj) {
    // animation component
    obj.components.animation = new paramEditor.LabelAndCheckbox("animation", false);
    obj.container.appendChild(obj.components.animation.domObj);

    obj.container.appendChild(document.createElement("br"));

    // delay component
    obj.components.delay = new paramEditor.LabelAndTextfield("delay", 250, "");
    obj.container.appendChild(obj.components.delay.domObj);

    // controls component
    obj.components.controls = new paramEditor.LabelAndCheckbox("controls", false);
    obj.container.appendChild(obj.components.controls.domObj);

    // auto component
    obj.components.auto = new paramEditor.LabelAndCheckbox("auto", false);
    obj.container.appendChild(obj.components.auto.domObj);

    // loop component
    obj.components.loop = new paramEditor.LabelAndCheckbox("loop", false);
    obj.container.appendChild(obj.components.loop.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // init component
    obj.components.init = new paramEditor.LabelAndTextfield("init", 250, "");
    obj.container.appendChild(obj.components.init.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente hacer un text area
    // do component
    obj.components.do = new paramEditor.LabelAndTextfield("do", 250, "");
    obj.container.appendChild(obj.components.do.domObj);

    obj.container.appendChild(document.createElement("br"));

    // ToDo: posiblemente cambiar a un componente que permita aumentar el area para editar su valor
    // while component
    obj.components.while = new paramEditor.LabelAndTextfield("while", 250, "");
    obj.container.appendChild(obj.components.while.domObj);

  }

  return paramEditor;
})(paramEditor || {});