/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var paramEditor = (function(paramEditor) {

  /**
   *
   */
  paramEditor.PanelListEdit = function(name, whitMenuList) {
    var self = this;
    this.components = {};
    this.container = document.createElement("div");
    this.container.setAttribute("class", "panelListEdit");

    var rnd = parseInt(Math.random()*1000);

    this.menu = document.createElement("select");
    this.menu.setAttribute("id", name+"_menu_"+rnd);
    this.setOptions([]);

    this.btnName = document.createElement("button");
    this.setButtonName(name);

    var btnContainer = document.createElement("div");
    btnContainer.setAttribute("class", "btnContainer");

    this.addBtn = document.createElement("button");
    this.addBtn.innerHTML = "+";
    this.addBtn.addEventListener("click", function(evt) { self.addElement(); });

    this.cloneBtn = document.createElement("button");
    this.cloneBtn.innerHTML = "*";
    this.cloneBtn.addEventListener("click", function(evt) { self.cloneElement(); });

    this.removeBtn = document.createElement("button");
    this.removeBtn.innerHTML = "-";
    this.removeBtn.addEventListener("click", function(evt) { self.removeElement(); });

    this.upBtn = document.createElement("button");
    this.upBtn.innerHTML = "⇧";
    this.upBtn.addEventListener("click", function(evt) { self.listUp(); });

    this.downBtn = document.createElement("button");
    this.downBtn.innerHTML = "⇩";
    this.downBtn.addEventListener("click", function(evt) { self.listDown(); });

    this.listPanel = document.createElement("div");
    this.listPanel.setAttribute("class", "listPanel");

    if (whitMenuList) {
      this.container.appendChild(this.menu);
    }
    else {
      this.listPanel.style.top = "60px";
    }
    this.container.appendChild(this.btnName);
    this.container.appendChild(btnContainer);
    this.container.appendChild(this.listPanel);
    btnContainer.appendChild(this.addBtn);
    btnContainer.appendChild(this.cloneBtn);
    btnContainer.appendChild(this.removeBtn);
    btnContainer.appendChild(this.upBtn);
    btnContainer.appendChild(this.downBtn);
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.setButtonName = function(name) {
    this.btnName.innerHTML = name;
  }


  /**
   *
   */
  paramEditor.PanelListEdit.prototype.setOptions = function(options) {
    // remove all previous children
    while (this.menu.firstChild) {
      this.menu.removeChild(this.menu.firstChild);
    }

    // add the new children
    var tmpOption;
    for (var i=0, l=options.length; i<l; i++) {
      tmpOption = document.createElement("option");
      tmpOption.setAttribute("value", options[i]);
      tmpOption.innerHTML = options[i];
      this.menu.appendChild(tmpOption);
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.addElement = function() {
    var self = this;

    // if the list have elements
    if (this.lastIndex >= 0) {
      this.divList[this.lastIndex].setAttribute("data-active", "false");
    }

    if (this.nameParam == "spaces") {
      var tmpElement = this.model.newSpaceObject([
        { name: "type", value: "R2" },
        { name: "id", value: "E"+ (this.divList.length+1) }
      ]);
    }
    else if (this.nameParam == "controls") {
      var tmpElement = this.model.newControlObject([
        { name: "type", value: "numeric" },
        { name: "gui", value: "spinner" },
        { name: "id", value: "n"+ (this.divList.length+1) }
      ]);
    }

    this.data[this.nameParam].push(tmpElement);

    var div = document.createElement("div");
    div.index = this.divList.length;
    div.innerHTML = tmpElement[this.nameElem] + "  «" + tmpElement[this.nameValue] + "»";
    div.addEventListener("click", function(evt) {
      self.divList[self.lastIndex].setAttribute("data-active", "false");
      self.lastIndex = this.index;
      this.setAttribute("data-active", "true");

      self.setPanelModelObj(this.index);
    });
    this.divList.push(div);

    this.listPanel.appendChild(div);

    self.setPanelModelObj(div.index);


    this.lastIndex = this.divList.length-1;
    this.divList[this.lastIndex].setAttribute("data-active", "true");
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.cloneElement = function() {
    var self = this;

    // if the list have elements
    if (this.lastIndex >= 0) {
      this.divList[this.lastIndex].setAttribute("data-active", "false");

      var tmpElement = clone(this.data[this.nameParam][this.lastIndex]);

      this.data[this.nameParam].push(tmpElement);

      var div = document.createElement("div");
      div.index = this.divList.length;
      div.innerHTML = tmpElement[this.nameElem] + "  «" + tmpElement[this.nameValue] + "»";
      div.addEventListener("click", function(evt) {
        self.divList[self.lastIndex].setAttribute("data-active", "false");
        self.lastIndex = this.index;
        this.setAttribute("data-active", "true");

        self.setPanelModelObj(this.index);
      });
      this.divList.push(div);

      this.listPanel.appendChild(div);

      self.setPanelModelObj(div.index);

      this.lastIndex = this.divList.length-1;
      this.divList[this.lastIndex].setAttribute("data-active", "true");
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.removeElement = function() {
    if (this.divList.length > 0) {
      // deactivate element
      this.divList[this.lastIndex].setAttribute("data-active", "false");

      // remove element from the dom list
      this.divList[this.lastIndex].parentNode.removeChild(this.divList[this.lastIndex]);

      // move elements
      for (var i=this.lastIndex, l=this.divList.length-1; i<l; i++) {
        this.divList[i] = this.divList[i+1];
        this.divList[i].index = i;
        this.data[this.nameParam][i] = this.data[this.nameParam][i+1];
      }

      this.divList.pop();
      this.data[this.nameParam].pop();

      if (this.lastIndex > this.divList.length-1) {
        this.lastIndex = this.divList.length-1;
      }
    }

    if (this.divList.length > 0) {
      this.setPanelModelObj(this.lastIndex);
      this.divList[this.lastIndex].setAttribute("data-active", "true");
    }
    // when dont have elements, hide the attributes
    else {
      this.editPanel.setModelObj({});
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.listUp = function() {
    var lastIndex = this.lastIndex;

    if (lastIndex > 0) {
      // swap elements in the dom
      this.listPanel.insertBefore(this.divList[lastIndex], this.divList[lastIndex-1])

      // swap elements in the divList
      var tmp = this.divList[lastIndex-1];
      this.divList[lastIndex-1] = this.divList[lastIndex];
      this.divList[lastIndex] = tmp;
      this.divList[lastIndex-1].index = lastIndex-1;
      this.divList[lastIndex].index = lastIndex;

      // swap elements in the model
      tmp = this.data[this.nameParam][lastIndex-1];
      this.data[this.nameParam][lastIndex-1] = this.data[this.nameParam][lastIndex];
      this.data[this.nameParam][lastIndex] = tmp;

      this.lastIndex--;
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.listDown = function() {
    var lastIndex = this.lastIndex;

    if (lastIndex < this.divList.length-1) {
      // swap elements in the dom
      this.listPanel.insertBefore(this.divList[lastIndex], this.divList[lastIndex+1].nextSibling)

      // swap elements in the divList
      var tmp = this.divList[lastIndex+1];
      this.divList[lastIndex+1] = this.divList[lastIndex];
      this.divList[lastIndex] = tmp;
      this.divList[lastIndex+1].index = lastIndex+1;
      this.divList[lastIndex].index = lastIndex;

      // swap elements in the model
      tmp = this.data[this.nameParam][lastIndex+1];
      this.data[this.nameParam][lastIndex+1] = this.data[this.nameParam][lastIndex];
      this.data[this.nameParam][lastIndex] = tmp;

      this.lastIndex++;
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.updatePresentation = function() {
    var list = this.data[this.nameParam];
    for (var i=0, l=this.divList.length; i<l; i++) {
      this.divList[i].innerHTML = list[i][this.nameElem] + "  «" + list[i][this.nameValue] + "»";
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.setModelObj = function(model, nameParam) {
    this.model = model;
    this.data = model.data;
    this.nameParam = nameParam;
    this.nameElem = (nameParam.match(/graphics/)) ? "type" : "id";
    this.nameValue = (nameParam.match(/graphics/)) ? "expression" : "type";

    var list  = this.data[nameParam];
    this.divList = [];
    this.lastIndex = 0;

    var div;
    var self = this;
    for (var i=0, l=list.length; i<l; i++) {
      div = document.createElement("div");
      div.index = i;
      div.innerHTML = list[i][this.nameElem] + "  «" + list[i][this.nameValue] + "»";
      div.addEventListener("click", function(evt) {
        self.divList[self.lastIndex].setAttribute("data-active", "false");
        self.lastIndex = this.index;
        this.setAttribute("data-active", "true");

        self.setPanelModelObj(this.index);
      });

      this.divList.push(div);

      this.listPanel.appendChild(div);
    }

    if (list.length > 0) {
      this.divList[0].setAttribute("data-active", "true");
      this.setPanelModelObj(0);
    }
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.setEditPanel = function(panel) {
    this.editPanel = panel;
  }

  /**
   *
   */
  paramEditor.PanelListEdit.prototype.setPanelModelObj = function(index) {
    this.editPanel.setModelObj(this.data[this.nameParam][index]);
  }


  /**
   *
   */
  function clone(obj) {
    var objClone = {};

    for (var propName in obj) {
      if (obj.hasOwnProperty(propName)) {
        objClone[propName] = obj[propName];
      }
    }

    return objClone;
  }


  return paramEditor;
})(paramEditor || {});
