/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var paramEditor = (function(paramEditor) {

  /**
   *
   */
  paramEditor.PanelSpaces = function(type) {
    this.components = {};
    this.container = document.createElement("div");
    this.container.setAttribute("class", "panel");

    // id component
    this.components.id = new paramEditor.LabelAndTextfield("id", 100, "E1");
    this.container.appendChild(this.components.id.domObj);

    // x component
    this.components.x = new paramEditor.LabelAndTextfield("x", 100, 0);
    this.container.appendChild(this.components.x.domObj);

    // y component
    this.components.y = new paramEditor.LabelAndTextfield("y", 100, 0);
    this.container.appendChild(this.components.y.domObj);

    // width component
    this.components.width = new paramEditor.LabelAndTextfield("width", 100, "100%");
    this.container.appendChild(this.components.width.domObj);

    // height component
    this.components.height = new paramEditor.LabelAndTextfield("height", 100, "100%");
    this.container.appendChild(this.components.height.domObj);

    // drawif component
    this.components.drawif = new paramEditor.LabelAndTextfield("drawif", 150, 1);
    this.container.appendChild(this.components.drawif.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // fixed component
    this.components.fixed = new paramEditor.LabelAndCheckbox("fixed", false);
    this.container.appendChild(this.components.fixed.domObj);

    // scale component
    this.components.scale = new paramEditor.LabelAndTextfield("scale", 100, 48);
    this.container.appendChild(this.components.scale.domObj);

    // Ox component
    this.components["O.x"] = new paramEditor.LabelAndTextfield("O.x", 100, 0);
    this.container.appendChild(this.components["O.x"].domObj);

    // Oy component
    this.components["O.y"] = new paramEditor.LabelAndTextfield("O.y", 100, 0);
    this.container.appendChild(this.components["O.y"].domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    // image component
    this.components.image = new paramEditor.LabelAndTextfield("image", 100, "");
    this.container.appendChild(this.components.image.domObj);

    // bg_display component
    this.components.bg_display = new paramEditor.LabelAndMenu("bg_display", ["topleft", "stretch", "patch", "center"], "topleft");
    this.container.appendChild(this.components.bg_display.domObj);

    // background component
    this.components.background = new paramEditor.LabelAndColor("background", "");
    this.container.appendChild(this.components.background.domObj);

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // net component
    this.components.net = new paramEditor.CheckboxAndLabelAndColor("net", true, "");
    this.container.appendChild(this.components.net.domObj);

    // net10 component
    this.components.net10 = new paramEditor.CheckboxAndLabelAndColor("net10", true, "");
    this.container.appendChild(this.components.net10.domObj);

    // axes component
    this.components.axes = new paramEditor.CheckboxAndLabelAndColor("axes", true, "");
    this.container.appendChild(this.components.axes.domObj);

    // text component
    this.components.text = new paramEditor.CheckboxAndLabelAndColor("text", true, "");
    this.container.appendChild(this.components.text.domObj);

    // numbers component
    this.components.numbers = new paramEditor.LabelAndCheckbox("numbers", false);
    this.container.appendChild(this.components.numbers.domObj);

    // x_axis component
    this.components.x_axis = new paramEditor.LabelAndTextfield("x_axis", 150, "");
    this.container.appendChild(this.components.x_axis.domObj);

    // y_axis component
    this.components.y_axis = new paramEditor.LabelAndTextfield("y_axis", 150, "");
    this.container.appendChild(this.components.y_axis.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // render component
    this.components.render = new paramEditor.LabelAndMenu("render", ["sort", "painter", "ray trace"], "sort");
    this.container.appendChild(this.components.render.domObj);

    // split component
    this.components.split = new paramEditor.LabelAndCheckbox("split", false);
    this.container.appendChild(this.components.split.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // sensitive_to_mouse_movements component
    this.components.sensitive_to_mouse_movements = new paramEditor.LabelAndCheckbox("sensitive_to_mouse_movements", false);
    this.container.appendChild(this.components.sensitive_to_mouse_movements.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for R2 & R3 spaces
    ////////////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////////////
    // next components are only for App & HTMLIfrae spaces
    ////////////////////////////////////////////////////////////////////////////////////
    // file component
    this.components.file = new paramEditor.LabelAndTextfield("file", 300, "");
    this.container.appendChild(this.components.file.domObj);
    ////////////////////////////////////////////////////////////////////////////////////
    // previous components are only for App & HTMLIfrae spaces
    ////////////////////////////////////////////////////////////////////////////////////
  }

  /**
   *
   */
  paramEditor.PanelSpaces.prototype.setEditPanel = function(panel) {
    this.editPanel = panel;
  }  

  /**
   *
   */
  paramEditor.PanelSpaces.prototype.setModelObj = function(objModel) {
    this.objModel = objModel;

    // traverse the values of the components to asign the object model
    for (var propName in this.components) {
      // verify the own properties of the object
      if (this.components.hasOwnProperty(propName)) {
        // show only the attributes of the object
        this.components[propName].domObj.style.display = (objModel[propName] != undefined) ? "inline-block" : "none";

        // verify the own properties of the object
        if (this.components[propName].setModelObj) {
          this.components[propName].setModelObj(objModel, this.editPanel);
        }
      }
    }
  }

  return paramEditor;
})(paramEditor || {});